@@ -1775,15 +1775,21 @@ module NumRu
         return
       end
 
+      # BACKUP CURRENT PARAMETERS
       itrsv = DCL::sgqtrn
       if itrsv <= 4
         ux1sv, ux2sv, uy1sv, uy2sv = DCL.sgqwnd
       elsif itrsv <= 9
         simfacsv, vxoffsv, vyoffsv = DCL.sgqsim
       else
+        ux1sv, ux2sv, uy1sv, uy2sv = DCL.sgqwnd
         simfacsv, vxoffsv, vyoffsv = DCL.sgqsim
-        plxsv, plysv, plrotsv = DCL.sgqmpl()
+        #plxsv, plysv, plrotsv = DCL.sgqmpl()
+        plxsv, plysv, plrotsv = DCL.umqcnt
+        lglobesv = DCL::umpget('lglobe')
+        grstxysv = %w(txmin txmax tymin tymax).map{|key| DCL::sgrget(key)}
       end
+      # END OF BACKUP
 
       vwidth = opt["vwidth"]
       vlength = opt["vlength"]
@@ -2348,11 +2354,15 @@ module NumRu
           DCL::grstrn(itrsv)
         ### TENTATIVE WORKAROUND (2012/2/9)
         elsif itrsv <= 34
+          DCL::grswnd(ux1sv, ux2sv, uy1sv, uy2sv)
           DCL.sgstrn(itrsv)
+          DCL.umpset('lglobe', lglobesv)
+          DCL::grstxy(*grstxysv)
+          DCL.umscnt(plxsv,plysv,plrotsv)
           DCL.umpfit
         ###
           DCL.sgssim(simfacsv,vxoffsv,vyoffsv)
-          DCL.sgsmpl(plxsv,plysv,plrotsv)
+          #DCL.sgsmpl(plxsv,plysv,plrotsv)
         end
         DCL::grstrf
 
