@@ -409,7 +409,13 @@
     ((|You know what it is.|))
 
 ---shape_coerce(other)
-    Like ((<coerce>)), but just changes shape without changing numeric type.
+    Like ((<coerce>)), but just changes shape without changing numeric type
+    by possibly inserting dimensions whose lengths are one (i.e.,
+    without changeing the total length of data).
+
+---shape_coerce_full(other)
+    Similar to ((<shape_coerce>)) but to return the gphyses having  
+    really the same shape with possible expansion by replication.
 
 ---transpose(*dims)
     Transpose.
@@ -767,6 +773,16 @@ module NumRu
 	 end
       end
 
+      def shape_coerce_full(other)
+        o, s = shape_coerce(other)
+        if o.length < s.length
+          o = GPhys.new( s.grid, o + NArray.new(o.typecode,*s.shape) )
+        elsif o.length > s.length
+          s = GPhys.new( o.grid, s + NArray.new(s.typecode,*o.shape) )
+        end
+        [o, s]
+      end
+
       def transpose(*dims)
 	grid = @grid.transpose(*dims)
 	data = @data.transpose(*dims)
