@@ -2,6 +2,12 @@ require "numru/gphys/gphys_netcdf_io"
 require "numru/gphys/gphys_grads_io"
 require "numru/gphys/gphys_grib_io"
 require "numru/gphys/gphys_nusdas_io"
+require "numru/gphys/gphys_gtool3_io"
+begin
+  require "numru/gphys/gphys_hdfeos5_io"
+rescue LoadError
+end
+  
 
 =begin
 =module NumRu::GPhys::IO
@@ -55,11 +61,11 @@ For example, (('GPhys::IO.open(file, name)')) simply calls
 ---parse_gturl(gturl)
     Parses GTOOL4-type URLs to specify path, variable name,
     and optionally subsets, whose format is 
-    (('path@varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]'))
+    (('path[@|/]varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]'))
 
     ARGUMENTS
     * gturl (String) GTOOL4 URL, whose format is
-      (('path@varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]'))
+      (('path[@|/]varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]'))
 
     RETURN VALUES
     * An Array consisting of [file, var, slice, cut_slice, thinning], where
@@ -80,6 +86,24 @@ For example, (('GPhys::IO.open(file, name)')) simply calls
     RETURN VALUE
     * a GPhys
 
+---str2gphys(str)
+    Open a GPhys from a slash("/")-separated String
+    such as "U.nc/U" and "U.nc".
+    * Aimed to help quick jobs with interactive sessions
+      -- This method do not handle a GPhys across multiple files.
+    * if the variable path is ommited such as "U.nc",
+      try to find the variable in it -- read the file and if 
+      only one variable is found, assume that is the 
+      variable specified; otherwise, an exception is raised.
+    * URL is accepted, but it's only thru NetCDF assuming OPeNDAP.
+
+    ARGUMENTS
+    * a String (file_path[/variable_path])
+      e.g. "U.nc/U", "U.nc", "http://.../U.nc/U"
+
+    RETURN VALUE
+    * a GPhys
+
 ==Module constants
 
 ---GTURLfmt
@@ -92,6 +116,15 @@ module NumRu
     module IO
       module_function
 
+      ## switches for io libraries
+      # switch grib library
+      def use_gphys_grib
+        VArrayGrib.use_gphys_grib
+      end
+      def use_rb_grib
+        VArrayGrib.use_rb_grib
+      end
+
       ## // module functions to be defined in specific IO modules -->
       def open(file, varname)
 	file2specific_module(file)::open(file, varname)
@@ -125,19 +158,37 @@ module NumRu
       GRADS = "GRADS"
       GRIB = "GRIB"
       NUSDAS = "NUSDAS"
+      He5    = "He5"
+      GTOOL3 = "GTOOL3"
       @@iomdl =     {NETCDF => GPhys::NetCDF_IO,
                      GRADS => GPhys::GrADS_IO,
                      GRIB => GPhys::Grib_IO,
-                     NUSDAS => GPhys::NuSDaS_IO}
+                     NUSDAS => GPhys::NuSDaS_IO,
+                     GTOOL3 => GPhys::Gtool3_IO}
       @@file_class = {NETCDF => NetCDF,
                      GRADS => GrADS_Gridded,
-                     GRIB => Grib,
-                     NUSDAS => NuSDaS}
+                     #GRIB => VArrayGrib.grib, # Hash is not dynamic: see below
+                     NUSDAS => NuSDaS,
+                     GTOOL3 => Gtool3}
+      def @@file_class.[](key)
+        if key == GRIB
+          VArrayGrib.grib
+        else
+          super
+        end
+      end
       @@nc_pattern = [/\.nc$/]
       @@grad_pattern = [/\.ctl$/]
       @@grib_pattern = [/\.grib$/, /\.grb$/]
       @@nus_pattern = [/\.nus$/]
 
+      @@has_he5 = defined?(HE5)
+      if @@has_he5
+        @@iomdl[He5] = GPhys::HE5_IO
+        @@file_class [He5] = HE5
+        @@he5_pattern = [/\.he5$/]
+      end
+
       def file2type(file)
  	case file
 	when Array, NArray
@@ -146,10 +197,12 @@ module NumRu
 	  return NETCDF
 	when GrADS_Gridded
 	  return GRADS
-	when Grib
+	when VArrayGrib.grib
 	  return GRIB
         when NuSDaS
           return NUSDAS
+        when Gtool3
+          return GTOOL3
 	when Regexp
 	  return NETCDF     # So far, only NetCDF_IO supports Regexp. 
 	when *@@nc_pattern
@@ -167,7 +220,18 @@ module NumRu
           return GRADS if GrADS_IO.is_a_GrADS?(file)
           return GRIB if Grib_IO.is_a_Grib?(file)
           return NUSDAS if NuSDaS_IO.is_a_NuSDaS?(file)
+          return GTOOL3 if Gtool3_IO.is_a_Gtool3?(file)
 	end
+        if @@has_he5
+          case file
+          when HE5, HE5Sw
+            return He5
+          when *@@he5_pattern
+            return He5
+          when String
+            return He5   if HE5_IO.is_a_HE5?(file)
+          end
+        end
         return nil
       end
 
@@ -179,7 +243,7 @@ module NumRu
 	@@file_class[ file2type(file) ]
       end
 
-      types = ['nc','grad','grib','nus']
+      types = ['nc','grad','grib','nus','he5']
       types.each{|c|
 	eval <<-EOS
         def add_#{c}_pattern(*regexps)
@@ -212,14 +276,17 @@ module NumRu
         file2specific_module(file).var_names_except_coordinates(file)
       end
 
-      GTURLfmt = "path@varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]"
+      GTURLfmt = "path[@|/]varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]"
 
       def parse_gturl(gturl)
 	if /(.*)@(.*)/ =~ gturl
 	  file = $1
 	  var = $2
+        elsif /(.*)\/(.*)/ =~ gturl
+          file = $1
+          var = $2
 	else
-	  raise "invalid URL: '@' between path & variable is not found\n\n" + 
+	  raise "invalid URL: '[@|/]' between path & variable is not found\n\n" + 
 	         "URL format: " + GTURLfmt
 	end
 	if /,/ =~ var
@@ -273,6 +340,42 @@ module NumRu
 	gp
       end   # def open_gturl
 
+      def str2gphys(str)
+
+        case str
+        when /^https?:\/\//
+          file_tester = Proc.new{|fname| NetCDF.open(fname) rescue false}
+        when
+          file_tester = Proc.new{|fname| File.exists?(fname)}
+        end
+        fname = str; vname = nil   # initial value
+        while fname
+          if file_tester.call(fname)
+            break
+          else
+            if /(.*)\/([^\/]+)/ =~ fname
+              fname = $1
+              if vname.nil?
+                vname = $2
+              else
+                vname = $2 + "/" + vname
+              end
+            else
+              raise "Not found: #{str}"
+            end
+          end
+        end
+        if vname.nil?
+          vns = var_names_except_coordinates(fname)
+          if vns.length==1
+            vname=vns.first 
+          else
+            raise "#{str} has multiple variables #{vns.inspect}. Specify one."
+          end
+        end
+        open(fname,vname)
+      end
+
     end      # module IO
   end      # class GPhys
 end      # module NumRu
@@ -281,6 +384,11 @@ end      # module NumRu
 if $0 == __FILE__
   include NumRu
 
+  puts "\n** test str2gphys **\n"
+  p GPhys::IO.str2gphys("../../../testdata/T.jan.nc/T")
+  p GPhys::IO.str2gphys("../../../testdata/T.jan.nc")
+  exit
+
   puts "\n** test NETCDF **\n"
 
   file = "../../../testdata/T.jan.nc"
