@@ -369,6 +369,26 @@ module NumRu
 
 	grid = Grid.new( *axes )
 
+        if (nms = convention::assoc_coord_names(data)) && !(nms.include?(varname))
+          if files.is_a?(NArray)
+            assoc_crds = nms.collect{|nm| 
+              index_files_assoc = [0]*(files.rank) 
+              tmp_assoc = GPhys::NetCDF_IO.open(files[0],nm)
+              tmp_assoc.rank.times{|i_assoc|
+                axes.length.times{|i_axes|
+                  if tmp_assoc.axis(i_assoc).name == axes[i_axes].name
+                    index_files_assoc[i_axes] = true
+                  end
+                }
+              }
+              GPhys::NetCDF_IO.open(files[*(index_files_assoc)],nm)
+            }
+          else
+            assoc_crds = nms.collect{|nm| GPhys::NetCDF_IO.open(files,nm)}
+          end
+          grid.set_assoc_coords(assoc_crds)
+        end
+
 	GPhys.new(grid,data)
       end
 
@@ -407,6 +427,12 @@ module NumRu
 	  }
 	  newaxes.push( newax )
 	}
+        if acnms = grid_or_gphys.assoccoordnames
+          acnms.each{|nm| 
+            acgp = grid_or_gphys.assoc_coord_gphys(nm)
+            VArrayNetCDF.write(file, acgp.data, nil, acgp.axnames)
+          }
+        end
 	Grid.new( *newaxes )
       end
 
@@ -417,7 +443,14 @@ module NumRu
       def write(file, gphys, name=nil)
         write_grid(file, gphys)
 	NetCDF_Conventions.add_history(file, "#{File.basename($0)} wrote "+gphys.name)
-	VArrayNetCDF.write(file, gphys.data, name, gphys.axnames)
+        data = gphys.data
+        if gphys.has_assoccoord?
+          if data.class != VArray
+            data = data.copy
+          end
+          data.put_att("coordinates",gphys.assoccoordnames.join(' '))
+        end
+	VArrayNetCDF.write(file, data, name, gphys.axnames)
 	nil
       end
 
