@@ -31,6 +31,7 @@ end
 ## <---/
 
 module NumRu
+class GPhys
   module GribUtils
     require "numru/gphys/grib_params"
     private
@@ -230,7 +231,7 @@ module NumRu
           len = file.read(3).to_uint3
           bds = GribBDS.new(obj,file.pos,len)
           obj.bds = bds
-          file.seek(len-3,IO::SEEK_CUR)
+          file.seek(len-3,::IO::SEEK_CUR)
 
           es = GribES.new(obj,file.read(4))
           unless es.grib?
@@ -1020,14 +1021,14 @@ module NumRu
       private
       def oct4
         return @oct4 if @oct4
-        @sgm.file.seek(@pos, IO::SEEK_SET)
+        @sgm.file.seek(@pos, ::IO::SEEK_SET)
         @oct4 = @sgm.file.read(1).to_uint1
         return @oct4
       end
       def oct14
         return nil if flag4==0
         return @oct14 if @oct14
-        @sgm.file.seek(@pos+10, IO::SEEK_SET)
+        @sgm.file.seek(@pos+10, ::IO::SEEK_SET)
         @oct14 = @sgm.file.read(1).to_uint1
         return @oct14
       end
@@ -1070,7 +1071,7 @@ module NumRu
         ebu = (8-(eb&7))/nstep unless ebu==0
         eb = (eb+7)/8
         return NArray.sfloat(0) if eb==sb
-        file.seek(pos+sb,IO::SEEK_SET)
+        file.seek(pos+sb,::IO::SEEK_SET)
         str = file.read(eb-sb)
         return get_bits(str,nstep,nbits,sbu,ebu,NArray::SFLOAT)
       end
@@ -1183,19 +1184,19 @@ module NumRu
       end
       def offset
         return @offset if @offset
-        @sgm.file.seek(@pos+3, IO::SEEK_SET)
+        @sgm.file.seek(@pos+3, ::IO::SEEK_SET)
         @offset = float_value(@sgm.file.read(4))
         return @offset
       end
       def efactor
         return @efactor if @efactor
-        @sgm.file.seek(@pos+1, IO::SEEK_SET)
+        @sgm.file.seek(@pos+1, ::IO::SEEK_SET)
         @efactor = 2.0**str2int2( @sgm.file.read(2) )
         return @efactor
       end
       def value(shape,*arg)
         nbits_unuse = oct4&15
-        @sgm.file.seek(@pos+7, IO::SEEK_SET)
+        @sgm.file.seek(@pos+7, ::IO::SEEK_SET)
         nbits_pack = @sgm.file.read(1).to_uint1
         if (nbits_pack%8) == 0
           nstep = 8
@@ -1280,7 +1281,7 @@ module NumRu
         else
           raise "not defined yet"
           n1 = @file.read(2).to_uint2
-          @sgm.file.seek(1, IO::SEEK_CUR)
+          @sgm.file.seek(1, ::IO::SEEK_CUR)
           str = @sgm.file.read(6)
           n2 = str.to_uint2
           p1 = str.to_uint2(2)
@@ -1406,6 +1407,8 @@ module NumRu
         return defined?(is) && is
       end
       def open(fname)
+        #warn "This grib library is obsoleted."
+        #warn "Use http://ruby.gfd-dennou.org/products/rb-grib/ instead."
         f = Grib.new(fname,"r")
         mbio = NumRu::MultibitIO.new(fname)
         f.parse(mbio)
@@ -1427,7 +1430,7 @@ module NumRu
       while true
         @file.read(1)=="\000" && next
         @file.eof? && break
-        @file.seek(-1, IO::SEEK_CUR)
+        @file.seek(-1, ::IO::SEEK_CUR)
         sgm = GribSegment.parse(@file, mbio)
         return nil unless sgm
         name = sgm.pds.sname
@@ -2015,6 +2018,7 @@ module NumRu
   end
 
 end
+end
 
 
 #####################################################
