@@ -231,7 +231,7 @@ module NumRu
 	    idxary = @index_array[mida]
 	    if idxary.length==1
 	       fst = idxary[0]
-	       len = step = 1
+	       len = stp = 1
 	       slicer = __make_regular_slicer((scl=mapping.collapsed),fst,len,stp)
 	       SubsetMapping1D.new0.initialize_regular(slicer,scl,fst,len,stp)
 	    else
