@@ -201,7 +201,6 @@ module NumRu
     # * date (Date or DateTime)
     # * units (Units or String) : units of the UNumeric to be created
     def self::from_date(date, units, calendar=nil)
-      date = date.to_datetime
       sunits = units.to_s
       /(.*) *since *(.*)/ =~ sunits
       if (!$1 or !$2)
@@ -218,7 +217,7 @@ module NumRu
         when nil, "gregorian", "standard"
           time = Units['days'].convert( date-since, tun )
         when "proleptic_gregorian"
-          since = since.new_start(Date::GREGORIAN) # convert to Proleptic Gregorian
+          since = DateTime.parse(UNumeric::before_date_parse($2),false,Date::GREGORIAN)
           time = Units['days'].convert( date-since, tun )
         when "noleap", "365_day"
           since_yday = since - DateTime.new(since.year,1,1) # day number of year (0..364)
@@ -237,9 +236,14 @@ module NumRu
           if date.day == 31
             raise("day=31 is specified, but calendar is #{calendar}.")
           end
+          if date.is_a?(DateTime)
+            date_hour,date_min,date_sec = date.hour,date.min,date.sec
+          else
+            date_hour,date_min,date_sec = 0,0,0
+          end
           days = (date.year-since.year)*360 + (date.mon-since.mon)*30 + 
-                 (date.day-since.day) + Rational(date.hour-since.hour,24) + 
-                 Rational(date.min-since.min,1440) + Rational(date.sec-since.sec,86400)
+                 (date.day-since.day) + Rational(date_hour-since.hour,24) + 
+                 Rational(date_min-since.min,1440) + Rational(date_sec-since.sec,86400)
           time = Units['days'].convert( days.to_f, tun )
         else
           #raise("Unrecognized calendar: #{calendar}")
