@@ -987,6 +987,15 @@ These methods returns a NArray (not a VArray).
 
       alias shape_current shape
 
+      ## < marshal dump/load >
+      def marshal_dump
+        [@name, @mapping, @varray, @ary, @attr]
+      end
+       
+      def marshal_load(ary)
+        @name, @mapping, @varray, @ary, @attr = *ary
+      end
+
       ## < private methods >
       private
       def __rubber_expansion( args )
